/* dxdir.c -- XDIR info of all files >= given date/time (default=01/01/80)

klidxdir -- <path> --+----------------------------------!
.                    !             **                   !
.                    +--- <dd/mm/yy> ---+---------------+
.                    !                  !               !
.                    !                  +--- <hh:mm> ---+ (24-hour clock)
.                    !                                  !
.                    +--------- today ------------------+ (00:00)
.         ** = mm/dd/yy if MMDDYY is #define'd during compile

  version 1.1 - display spaces (not tabs) for syntax-diagram
			  - display file-time next to the date

		  1.2 - select based on time also
			  - new keyword "today"
			  - define MMDDYY for US-date-format
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dir.h>
#include <dos.h>

#define COPYRIGHT "(c) KLI Software Consultants (Amsterdam) B.V."
#define VERBANNER "*** KLIDXDIR - Xtended Dir. Report based on Date, Version DXDIR1.2 ***"

#define DOTS(offset) for (i=offset; i<78; i++) pr_line[i] = '.'

int  e_flags;	/* to see if the entered name was a directory name */
unsigned int f_dd,   f_mm,   f_yy,	  /* to compare the entered & file dates */
			 e_dd=1, e_mm=1, e_yy=80; /* default 'entered' date 01/01/80) */
unsigned int f_hh,   f_mn,            /* -- ditto times */
			 e_hh=0, e_mn=0;
int ok;								  /* if file-date/time was >= entered values */

int d31[] = {0,1,0,1,0,1,0,1,1,0,1,0,1,0}; /* for date-validation     */
unsigned int dt;						   /* for date conversion */
unsigned int tm, f_hh, f_mn, f_ss;		   /* to get the time */

struct date today;	/* for getdate() for "today" */
char pr_line[81]; 	/* our line buffer */
long int tot_size=0;
long int files=0;

void pr(void) /* print a line from pr_line */
{	int i;

	for (i=0; i<78; i++)
		if (pr_line[i] == 0)
			pr_line[i] = ' ';

	printf("%s\n", pr_line);

	for (i=0; i<78; i++)
		pr_line[i] = ' ';

} /* pr */

void get_fullname(char *dir, char *fullname) /* get the full expanded dir. */
{	char spl_drive[MAXDRIVE] = "",			 /* name using a possibly      */
		 spl_dir  [MAXDIR] = "",			 /* partial dir.name and the   */
		 spl_file [MAXFILE],				 /* current directory name */
		 spl_ext  [MAXEXT];
	int  len_drive, len_dir, i, j;

	e_flags = fnsplit(dir, spl_drive, spl_dir, spl_file, spl_ext);

	len_drive = strlen(spl_drive);
	len_dir   = strlen(spl_dir);
	if (len_drive == 0 || len_dir == 0)
	{	if (len_drive == 0) /* no drive specified */
		{	j = getdisk(); /* 0=A, 1=B, ... */
			spl_drive[0] = 'A' + j;
			spl_drive[1] = ':';
			spl_drive[2] = '\0';
		}
		if (len_dir == 0) /* no dir. specified -- get current dir */
		{	j = toupper(spl_drive[0]) - 'A' + 1; /* 1=A, 2=B, ... */
			spl_dir[0]   = '\\';
			getcurdir(j, &spl_dir[1]);
			j = strlen(spl_dir);
			if (spl_dir[1] != 0)
				spl_dir[j++] = '\\';
			spl_dir[j]   = '\0';
		}
	} /* end of retrieving unspecified drive/directory names */

	fnmerge(fullname, spl_drive, spl_dir, spl_file, spl_ext);

} /* get_fullname */

void get_filename(char *dir, char *filename) /* get the last-level file-   */
{	char spl_file [9], 						 /* name (name.ext) using a    */
		 spl_ext  [4];			 			 /* possibly full directory    */
											 /* name */
	fnsplit(dir, NULL, NULL, spl_file, spl_ext);
	fnmerge(filename, NULL, NULL, spl_file, spl_ext);

} /* get_filename */

char spl_drive[MAXDRIVE],
	 spl_dir  [MAXDIR];

void kli_xdir(char *dir, int indent_level, int level)
{   /*	the main  procedure of this program:
		it expands one directory name and prints the relevant info
		for all files & directories under that directory, calling itself
		recursively for each sub-, sub-sub-,... directory underneath
	*/
	char fullname[MAXPATH];
	char filename[13];
	char far *dta;
	struct ffblk ffblk;
	int i, j, x;

	strcpy(fullname, dir);

	if ((level == 0) && (e_flags & WILDCARDS)) /* strip away the wildcard info for printing */
	{	fnsplit(dir, spl_drive, spl_dir, NULL, NULL);
		fnmerge(dir, spl_drive, spl_dir, NULL, NULL);
	}
	else /* we have to expand a directory name */
	{	i = strlen(fullname) - 1;
		if (fullname[i] == '\\')		/* last char is a backslash ? */
			strcat(fullname, "*.*");	/* yes, append a "*.*" */
		else
			strcat(fullname, "\\*.*");	/* no,  append a backslash & then "*.*" */
	}

	if (findfirst(fullname, &ffblk, FA_RDONLY | FA_HIDDEN |
									FA_SYSTEM | FA_DIREC  |
									FA_ARCH)) /* none found */
	{	sprintf(pr_line, "no files found in the directory %s", fullname);
		pr();
		return;
	}

	dta = getdta();

	x = indent_level+4;
	do
		if (ffblk.ff_name[0] != '.')
			if (ffblk.ff_attrib == FA_DIREC)
			{	strcpy(fullname, dir);
				i = strlen(fullname) - 1;
				if (fullname[i] != '\\')
					strcat(fullname, "\\");
				strcat(fullname, ffblk.ff_name);
				kli_xdir(fullname, x, level+1);
			}
			else
			{   dt   = ffblk.ff_fdate;
				f_dd = (dt << 11) >> 11;
				f_mm = (dt << 7) >> 12;
				f_yy = (dt >> 9) + 80;
				ok = 0;
				if (e_yy < f_yy)
					ok = 1;
				else if (e_yy == f_yy)
					if (e_mm < f_mm)
						ok = 1;
					else if (e_mm == f_mm)
						if (e_dd < f_dd)
							ok = 1;
						else
							if (e_dd == f_dd)	/* special case */
								ok = 2;			/* need to check the time */
				if (ok > 0)
				{   tm = ffblk.ff_ftime;
					f_ss = ((tm << 11) >> 11)*2;
					f_mn = (tm << 5) >> 10;
					f_hh = (tm >> 11);
					if (ok == 2) /* date matches exactly -- check time */
					{	ok = 0;
						if (e_hh < f_hh)
							ok = 1;
						else if (e_hh == f_hh)
							if (e_mn <= f_mn)
								ok = 1;
					}
				}
				if (ok == 1)
				{	files++;
					strcpy(fullname, dir);
					i = strlen(fullname) - 1;
					if (fullname[i] != '\\')
						strcat(fullname, "\\");
					strcat(fullname, ffblk.ff_name);
					DOTS(x);
					sprintf(&pr_line[4], "%s", fullname);
#ifdef MMDDYY
					sprintf(&pr_line[53], "%02d/%02d/%02d", f_mm, f_dd, f_yy);
#else
					sprintf(&pr_line[53], "%02d/%02d/%02d", f_dd, f_mm, f_yy);
#endif
					sprintf(&pr_line[62], "%02d:%02d:%02d", f_hh, f_mn, f_ss);
					sprintf(&pr_line[70], "%8ld", ffblk.ff_fsize);
					pr();
					tot_size+= ffblk.ff_fsize;
				}
				setdta (dta);
			}
	while (!findnext(&ffblk));

	if (level == 0) /* end of outer level */
	{   pr();
		DOTS(x);
		sprintf(&pr_line[x],  "Total Space Occupied (%ld files)", files);
		sprintf(&pr_line[70], "%8ld", tot_size);
		pr();
	}

	return;

} /* kli_xdir */

char dir_name[MAXPATH];

void main(int argc, char *argv[])
{   int i;

	printf(COPYRIGHT "\n");
	printf(VERBANNER "\n");

	if (argc == 1)
	{	printf("               (an XDIR info of all files >= given date/time)\n");
		printf("                default = 01/01/80 00:00\n");
		printf("syntax:\n");
		printf("  --- <path> ----+----------------------------------!\n");
		printf("                 !                                  !\n");
#ifdef MMDDYY
		printf("                 +--- <mm/dd/yy> ---+---------------+\n");
#else
		printf("                 +--- <dd/mm/yy> ---+---------------+\n");
#endif
		printf("                 !                  !               !\n");
		printf("                 !                  +--- <hh:mm> ---+ (24-hour clock)\n");
		printf("                 !                                  !\n");
		printf("                 +------------- today --------------+ (00:00)\n\n");
		printf("where <path> can be either a directory name or a 'wildcard'\n");
		return;
	}

	get_fullname(argv[1], dir_name);

	if (argc > 2) /* date/time or "today" entered */
		if (stricmp(argv[2], "today") == 0)	/* set up current date */
		{	getdate(&today);
			e_yy = today.da_year - 1900;
			e_mm = today.da_mon;
			e_dd = today.da_day;
		}
		else								/* probably date -- validate */
#ifdef MMDDYY
		{	if (sscanf(argv[2], "%d/%d/%d", &e_mm, &e_dd, &e_yy) != 3)
			{	printf("date should be entered as 'mm/dd/yy': %s", argv[2]);
#else
		{	if (sscanf(argv[2], "%d/%d/%d", &e_dd, &e_mm, &e_yy) != 3)
			{	printf("date should be entered as 'dd/mm/yy': %s", argv[2]);
#endif
				return;
			}
			if ((e_mm > 12) || (e_mm == 0))
			{	printf("month should be 1 thru 12: %d", e_mm);
				return;
			}
			if ((e_dd == 0) || (e_dd > 31) ||
				((e_dd == 31) && (d31[e_mm] != 1)) ||
				((e_dd == 30) && (e_mm == 2)))
			{	printf("invalid day: %d (month=%2d)", e_dd, e_mm);
				return;
			}
			if ((e_mm == 2) && (e_dd == 29))
				if ((e_yy == 0) || (e_yy % 4 != 0))
				{	printf("not a leap year: %2d", e_yy);
					return;
				}

			if (argc == 4) /* we have the time also */
			{	if (sscanf(argv[3], "%d:%d", &e_hh, &e_mn) != 2)
				{	printf("time should be entered as 'hh:mn' (24-hour clock): %s", argv[3]);
					return;
				}
				if ((e_hh > 23) || (e_mn > 59))
				{	printf("invalid time (24-hour clock): %s", argv[3]);
					return;
				}
			}
		} /* date/time specified */

	sprintf(pr_line, "XDir details (>= %02d/%02d/%02d %02d:%02d) for %s\n",
#ifdef MMDDYY
					 e_mm, e_dd, e_yy, e_hh, e_mn, dir_name);
#else
					 e_dd, e_mm, e_yy, e_hh, e_mn, dir_name);
#endif
	pr();

	kli_xdir(dir_name, 1, 0);
}